####################################################################
# Below various summary statistics are calculated and outputed to an .XLSX file. We use the XLConnect package to write the summary statistics to the .xlsx file.
# The file will be written to current working directory, if you want to change the location of the file please specify the location followed by file name.
# to get the current working directory type getwd() in your R console. You can also change the format of the file to an xls file by changing the file extension.
# Summary statistics will be outputted as separate tabs in the xls or xlsx file.
####################################################################

if (!(require(xlsx))) install.packages ("xlsx")
if (!(require(data.table))) install.packages ("data.table")
if (!(require(dplyr))) install.packages ("dplyr")

### Disable scientific notation
options(scipen=999)

### Define column headers and classes
lppub_column_names <- c(
  "LOAN_ID", "ORIG_CHN", "SELLER", "orig_rt", "orig_amt",
  "orig_trm", "oltv", "ocltv", "num_bo", "dti",
  "CSCORE_B", "FTHB_FLG", "purpose", "PROP_TYP", "NUM_UNIT",
  "occ_stat", "state", "zip_3", "mi_pct", "CSCORE_C",
  "relo_flg", "MI_TYPE", "AQSN_DTE", "ORIG_DTE", "FRST_DTE",
  "LAST_RT", "LAST_UPB", "msa", "FCC_COST", "PP_COST",
  "AR_COST", "IE_COST", "TAX_COST", "NS_PROCS", "CE_PROCS",
  "RMW_PROCS", "O_PROCS", "repch_flag", "LAST_ACTIVITY_DATE",
  "LPI_DTE", "FCC_DTE", "DISP_DTE", "SERVICER", "F30_DTE",
  "F60_DTE", "F90_DTE", "F120_DTE", "F180_DTE", "FCE_DTE",
  "F180_UPB", "FCE_UPB", "F30_UPB", "F60_UPB", "F90_UPB",
  "MOD_FLAG", "FMOD_DTE", "FMOD_UPB", "MODIR_COST", "MODFB_COST",
  "MODFG_COST", "MODTRM_CHNG", "MODUPB_CHNG", "z_num_periods_120", "F120_UPB",
  "CSCORE_MN", "ORIG_VAL", "LAST_DTE", "LAST_STAT", "COMPLT_FLG",
  "INT_COST", "PFG_COST", "NET_LOSS", "NET_SEV", "MODTOT_COST"
)

lppub_column_classes <- c(
  "character", "character", "character", "numeric", "numeric",
  "numeric", "numeric", "numeric", "numeric", "numeric",
  "numeric", "character", "character", "character", "numeric",
  "character", "character", "character", "numeric", "numeric",
  "character", "character", "character", "character", "character",
  "numeric", "numeric", "character", "numeric", "numeric",
  "numeric", "numeric", "numeric", "numeric", "numeric",
  "numeric", "numeric", "character", "character",
  "character", "character", "character", "character", "character",
  "character", "character", "character", "character", "character",
  "numeric", "numeric", "numeric", "numeric", "numeric",
  "character", "character", "numeric", "numeric", "numeric",
  "numeric", "character", "character", "numeric", "numeric",
  "numeric", "numeric", "character", "character", "character",
  "numeric", "numeric", "numeric", "numeric", "numeric"
)

### Read in .csv files generated by LPPUB_StatFile and LPPUB_StatFile_Production
starting_file <- 0 #0 starts on 2000Q1
ending_file <- 81 #81 ends on 2020Q2

for (file_number in starting_file:ending_file){

	#Set up file names
	fileYear <- file_number %/% 4
	if(nchar(fileYear) == 1){
	  fileYear <- paste0('0', fileYear)
	}
	fileYear <- paste0('20', fileYear)
	fileQtr <- (file_number %% 4) + 1
	fileQtr <- paste0('Q',fileQtr)
	FileName <- paste0(fileYear, fileQtr, "_stat.csv")
	
	#Load files
	if(file_number == starting_file){
	  lppub_files <- fread(FileName, sep = ",", col.names = lppub_column_names, colClasses = lppub_column_classes, na.strings = "NULL")
	} else {
	  lppub_files <- rbind(lppub_files, fread(FileName, sep = ",", col.names = lppub_column_names, colClasses = lppub_column_classes, na.strings = "NULL"))
	}

}

### Add ORIG_YR, DISP_YR, DFLT_UPB, TOT_COST, TOT_PROCS and LIQ_EXP to the data to help compute summary fields
lppub_files <- lppub_files %>%
  mutate(
    ORIG_YR = substr(ORIG_DTE,1,4),
    DISP_YR = substr(DISP_DTE,1,4),
    COMPLT_FLG = as.numeric(COMPLT_FLG),
    COMPLT_FLG = if_else(COMPLT_FLG != 1, 0, 1),
    DFLT_UPB = LAST_UPB * COMPLT_FLG,
    TOT_COST = FCC_COST+PP_COST+AR_COST+IE_COST+TAX_COST+DFLT_UPB+INT_COST,
    TOT_PROCS = NS_PROCS+CE_PROCS+RMW_PROCS+O_PROCS,
    LIQ_EXP = FCC_COST+PP_COST+AR_COST+IE_COST+TAX_COST
  )

### Prepare the Acquisition Stats summary
aqsn_stat <- lppub_files %>%
  group_by(ORIG_YR) %>%
  summarize(
    LOAN_COUNT = n(),
    SUM_orig_amt = sum(orig_amt)/1000000,
    AVG_orig_amt = mean(orig_amt),
    CSCORE_B = sum(CSCORE_B*orig_amt, na.rm = TRUE)/sum(if_else(is.na(CSCORE_B), 0, orig_amt)),
    CSCORE_C = sum(CSCORE_C*orig_amt, na.rm = TRUE)/sum(if_else(is.na(CSCORE_C), 0, orig_amt)),
    oltv = sum(oltv*orig_amt, na.rm = TRUE)/sum(if_else(is.na(oltv), 0, orig_amt)),
    ocltv = sum(ocltv*orig_amt, na.rm = TRUE)/sum(if_else(is.na(ocltv), 0, orig_amt)),
    dti = sum(dti*orig_amt, na.rm = TRUE)/sum(if_else(is.na(dti), 0, orig_amt)),
    orig_rt = sum(orig_rt*orig_amt, na.rm = TRUE)/sum(if_else(is.na(orig_rt), 0, orig_amt))
  )

# Calculate totals
aqsn_total <- lppub_files %>%
  summarize(
    LOAN_COUNT = n(),
    SUM_orig_amt = sum(orig_amt)/1000000,
    AVG_orig_amt = mean(orig_amt),
    CSCORE_B = sum(CSCORE_B*orig_amt, na.rm = TRUE)/sum(if_else(is.na(CSCORE_B), 0, orig_amt)),
    CSCORE_C = sum(CSCORE_C*orig_amt, na.rm = TRUE)/sum(if_else(is.na(CSCORE_C), 0, orig_amt)),
    oltv = sum(oltv*orig_amt, na.rm = TRUE)/sum(if_else(is.na(oltv), 0, orig_amt)),
    ocltv = sum(ocltv*orig_amt, na.rm = TRUE)/sum(if_else(is.na(ocltv), 0, orig_amt)),
    dti = sum(dti*orig_amt, na.rm = TRUE)/sum(if_else(is.na(dti), 0, orig_amt)),
    orig_rt = sum(orig_rt*orig_amt, na.rm = TRUE)/sum(if_else(is.na(orig_rt), 0, orig_amt))
  )

aqsn_total <- cbind('Total', aqsn_total)
colnames(aqsn_total) <- colnames(aqsn_stat)

# Append totals to yearly stats  
aqsn_stat <- rbind(aqsn_stat, aqsn_total)

colnames(aqsn_stat) <- c("Origination Year", "Loan Count", "Total Orig. UPB ($M)", "Avg. Orig UPB ($M)",
                         "Borrower Credit Score", "Co-Borrower Credit Score", "LTV Ratio", "CLTV Ratio", "DTI", "Note Rate")

### Prepare the Performance Stats summary
perf_stat <- lppub_files %>%
  group_by(ORIG_YR) %>%
  summarize(
    LOAN_COUNT = n(),
    SUM_orig_amt = sum(orig_amt)/1000000,
    ACTIVE_COUNT = sum(if_else(LAST_STAT %in% c('P','R','N','F','S','T','L'), 0, 1)),
    ACTIVE_UPB = sum(if_else(LAST_STAT %in% c('P','R','N','F','S','T','L'), 0, 1) * LAST_UPB, na.rm = TRUE) / 1000000,
    PREPAY = sum(if_else(LAST_STAT == 'P', 1, 0)),
    RPCH = sum(if_else(LAST_STAT == 'R', 1, 0)),
	SS = sum(if_else(LAST_STAT == 'S', 1, 0)),
	TPS = sum(if_else(LAST_STAT == 'T', 1, 0)),
	REO = sum(if_else(LAST_STAT == 'F', 1, 0)),
    NPL = sum(if_else(LAST_STAT == 'N', 1, 0)),
    RPL = sum(if_else(LAST_STAT == 'L', 1, 0)),
    MOD = sum(if_else(!is.na(FMOD_DTE), 1, 0)),
    D180_UPB = sum(FCE_UPB, na.rm = TRUE) / 1000000,
    FCE_RT = sum(FCE_UPB, na.rm = TRUE)/sum(orig_amt, na.rm = TRUE),
    DFLT_UPB = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
    NET_LOSS_RT = sum(NET_LOSS, na.rm = TRUE) / sum(orig_amt, na.rm = TRUE)
  )

# Calculate totals
perf_total <- lppub_files %>%
  summarize(
    LOAN_COUNT = n(),
    SUM_orig_amt = sum(orig_amt)/1000000,
    ACTIVE_COUNT = sum(if_else(LAST_STAT %in% c('P','R','N','F','S','T','L'), 0, 1)),
    ACTIVE_UPB = sum(if_else(LAST_STAT %in% c('P','R','N','F','S','T','L'), 0, 1) * LAST_UPB, na.rm = TRUE) / 1000000,
    PREPAY = sum(if_else(LAST_STAT == 'P', 1, 0)),
    RPCH = sum(if_else(LAST_STAT == 'R', 1, 0)),
	SS = sum(if_else(LAST_STAT == 'S', 1, 0)),
	TPS = sum(if_else(LAST_STAT == 'T', 1, 0)),
	REO = sum(if_else(LAST_STAT == 'F', 1, 0)),
    NPL = sum(if_else(LAST_STAT == 'N', 1, 0)),
    RPL = sum(if_else(LAST_STAT == 'L', 1, 0)),
    MOD = sum(if_else(!is.na(FMOD_DTE), 1, 0)),
    D180_UPB = sum(FCE_UPB, na.rm = TRUE) / 1000000,
    FCE_RT = sum(FCE_UPB, na.rm = TRUE)/sum(orig_amt, na.rm = TRUE),
    DFLT_UPB = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
    NET_LOSS_RT = sum(NET_LOSS, na.rm = TRUE) / sum(orig_amt, na.rm = TRUE)
  )

perf_total <- cbind('Total', perf_total)
colnames(perf_total) <- colnames(perf_stat)

# Append totals to yearly stats
perf_stat <- rbind(perf_stat, perf_total)

colnames(perf_stat) <- c("Origination Year", "Loan Count", "Total Orig. UPB ($M)", "Loan Count (Active)",
                         "Active UPB ($M)", "Prepaid (01)", "Repurchased (06)", "Short Sale (03)", "Third Party Sale (02)",
                         "REO (09)", "Non-Performing Loan Sale (15)", "Re-Performing Loan Sale (16)","Mod Loan Count", "D180 UPB", 
                         "D180% of Orig. UPB", "Default UPB", "Loss Rate (%)")

### Prepare the Disposition Loss summary
# Aggregate years 2000 through 2005
disp_table <- lppub_files %>%
  mutate(
    DISP_YR = case_when(
      DISP_YR == '2000' ~ '2000-2005',
      DISP_YR == '2001' ~ '2000-2005',
      DISP_YR == '2002' ~ '2000-2005',
      DISP_YR == '2003' ~ '2000-2005',
      DISP_YR == '2004' ~ '2000-2005',
      DISP_YR == '2005' ~ '2000-2005',
      TRUE ~ DISP_YR
    )
  )

disp_loss <- disp_table %>%
  filter(!is.na(DISP_YR)) %>%
  group_by(DISP_YR) %>%
  summarize(
    DFLT_UPB_SUM = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
    INT_COST = sum(INT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    LIQ_EXP = sum(FCC_COST+PP_COST+AR_COST+IE_COST+TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    FCC_COST = sum(FCC_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    PP_COST = sum(PP_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    AR_COST = sum(AR_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    IE_COST = sum(IE_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TAX_COST = sum(TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_COST = sum(TOT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NS_PROCS = sum(NS_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    CE_PROCS = sum(CE_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    RMW_PROCS = sum(RMW_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    O_PROCS = sum(O_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_PROCS = sum(TOT_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_SEV = sum(NET_LOSS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_LOSS = sum(NET_LOSS, na.rm = TRUE)/1000000
  )

# Calculate totals
disp_total <- disp_table %>%
  summarize(
    DFLT_UPB_SUM = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
    INT_COST = sum(INT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    LIQ_EXP = sum(FCC_COST+PP_COST+AR_COST+IE_COST+TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    FCC_COST = sum(FCC_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    PP_COST = sum(PP_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    AR_COST = sum(AR_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    IE_COST = sum(IE_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TAX_COST = sum(TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_COST = sum(TOT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NS_PROCS = sum(NS_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    CE_PROCS = sum(CE_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    RMW_PROCS = sum(RMW_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    O_PROCS = sum(O_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_PROCS = sum(TOT_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_SEV = sum(NET_LOSS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_LOSS = sum(NET_LOSS, na.rm = TRUE)/1000000
  )

disp_total <- cbind('Total', disp_total)
colnames(disp_total) <- colnames(disp_loss)

# Append totals to yearly stats
disp_loss <- rbind(disp_loss, disp_total)

# Transpose the data frame
disp_loss_transpose <- t(disp_loss)
disp_loss_transpose <- disp_loss_transpose[2:nrow(disp_loss_transpose),2:ncol(disp_loss_transpose)]
colnames(disp_loss_transpose) <- c(disp_loss$DISP_YR[2:ncol(disp_loss_transpose)], 'Total')

### Prepare the Origination Loss summary
# Aggregate years 1999 through 2004
orig_table <- lppub_files %>%
  mutate(
    ORIG_YR = case_when(
      ORIG_YR == '1999' ~ '1999-2004',
      ORIG_YR == '2000' ~ '1999-2004',
      ORIG_YR == '2001' ~ '1999-2004',
      ORIG_YR == '2002' ~ '1999-2004',
      ORIG_YR == '2003' ~ '1999-2004',
      ORIG_YR == '2004' ~ '1999-2004',
      TRUE ~ ORIG_YR
    )
  )

orig_loss <- orig_table %>%
  group_by(ORIG_YR) %>%
  summarize(
    DFLT_UPB_SUM = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
	DLFT_RT = sum(DFLT_UPB, na.rm = TRUE)/sum(orig_amt, na.rm = TRUE),
    INT_COST = sum(INT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    LIQ_EXP = sum(LIQ_EXP, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    FCC_COST = sum(FCC_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    PP_COST = sum(PP_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    AR_COST = sum(AR_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    IE_COST = sum(IE_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TAX_COST = sum(TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_COST = sum(TOT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NS_PROCS = sum(NS_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    CE_PROCS = sum(CE_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    RMW_PROCS = sum(RMW_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    O_PROCS = sum(O_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_PROCS = sum(TOT_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_SEV = sum(NET_LOSS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_LOSS = sum(NET_LOSS, na.rm = TRUE)/1000000
  )

# Calculate totals
orig_total <- orig_table %>%
  summarize(
    DFLT_UPB_SUM = sum(DFLT_UPB, na.rm = TRUE) / 1000000,
	DLFT_RT = sum(DFLT_UPB, na.rm = TRUE)/sum(orig_amt, na.rm = TRUE),
    INT_COST = sum(INT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    LIQ_EXP = sum(LIQ_EXP, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    FCC_COST = sum(FCC_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    PP_COST = sum(PP_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    AR_COST = sum(AR_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    IE_COST = sum(IE_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TAX_COST = sum(TAX_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_COST = sum(TOT_COST, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NS_PROCS = sum(NS_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    CE_PROCS = sum(CE_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    RMW_PROCS = sum(RMW_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    O_PROCS = sum(O_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    TOT_PROCS = sum(TOT_PROCS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_SEV = sum(NET_LOSS, na.rm = TRUE)/sum(DFLT_UPB, na.rm = TRUE),
    NET_LOSS = sum(NET_LOSS, na.rm = TRUE)/1000000
  )

orig_total <- cbind('Total', orig_total)
colnames(orig_total) <- colnames(orig_loss) 

# Append totals to yearly stats
orig_loss <- rbind(orig_loss, orig_total)

# Transpose the data frame
orig_loss_transpose <- t(orig_loss)
orig_loss_transpose <- orig_loss_transpose[2:nrow(orig_loss_transpose),1:ncol(orig_loss_transpose)]
colnames(orig_loss_transpose) <- c(orig_loss$ORIG_YR[1:ncol(orig_loss_transpose)])

# Build workbook
wb <- createWorkbook(type="xlsx")
aqsnsheet <-createSheet(wb, sheetName = "AqsnProfile")
perfsheet <-createSheet(wb, sheetName = "PerfProfile")
lossDsheet <-createSheet(wb, sheetName = "DLoss")
lossOsheet <-createSheet(wb, sheetName = "OLoss")

addDataFrame(aqsn_stat, aqsnsheet, startRow=3, startColumn=2)
addDataFrame(perf_stat, perfsheet, startRow=3, startColumn=2)
addDataFrame(disp_loss_transpose, lossDsheet, startRow=4, startColumn=2)
addDataFrame(orig_loss_transpose, lossOsheet, startRow=4, startColumn=2)
saveWorkbook(wb, "Summary Statistics.xlsx")

####################################################################
# End of Summary Statistics production
####################################################################